<?php

namespace Litepie\Message\Actions;

use Illuminate\Support\Carbon;
use Illuminate\Support\Str;
use Litepie\Actions\Concerns\AsAction;
use Litepie\Actions\Traits\LogsActions;
use Litepie\Notification\Traits\SendNotification;
use Litepie\Message\Models\Message;


class MessageAction
{
    use AsAction;
    use LogsActions;
    use SendNotification;

    private $model;

    public function handle(string $action, Message $message, array $request = [])
    {
        $this->model = $message;

        $function = Str::camel($action);
        event('litepie.message.message.action.' . $action . 'ing', [$message]);
        $data =  $this->$function($message, $request);
        event('litepie.message.message.action.' . $action . 'ed', [$message]);

        $this->logsAction();
        $this->notify();
        return $data;
    }


    public function store(Message $message, array $request)
    {
        $attributes = $request;
        $attributes['user_id'] = user_id();
        $attributes['user_type'] = user_type();
        $message = $message->create($attributes);
        return $message;
    }

    public function update(Message $message, array $request)
    {
        $attributes = $request;
        $message->update($attributes);
        return $message;
    }

    public function destroy(Message $message, array $request)
    {
        $message->delete();
        return $message;
    }

    public function copy(Message $message, array $request)
    {
        $count = $request['count'] ?: 1;

        if ($count == 1) {
            $message = $message->replicate();
            $message->created_at = Carbon::now();
            $message->save();
            return $message;
        }

        for ($i = 1; $i <= $count; $i++) {
            $new = $message->replicate();
            $new->created_at = Carbon::now();
            $new->save();
        }

        return $message;
    }


}
