<?php

namespace Litepie\Message\Policies;

use Litepie\User\Interfaces\UserPolicyInterface;
use Litepie\Message\Models\Message;

class MessagePolicy
{


    /**
     * Determine if the given user can view the message.
     *
     * @param UserPolicyInterface $authUser
     * @param Message $message
     *
     * @return bool
     */
    public function view(UserPolicyInterface $authUser, Message $message)
    {
        if ($authUser->canDo('message.message.view') && $authUser->isAdmin()) {
            return true;
        }

        return $message->user_id == user_id() && $message->user_type == user_type();
    }

    /**
     * Determine if the given user can create a message.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function create(UserPolicyInterface $authUser)
    {
        return  $authUser->canDo('message.message.create');
    }

    /**
     * Determine if the given user can update the given message.
     *
     * @param UserPolicyInterface $authUser
     * @param Message $message
     *
     * @return bool
     */
    public function update(UserPolicyInterface $authUser, Message $message)
    {
        if ($authUser->canDo('message.message.edit') && $authUser->isAdmin()) {
            return true;
        }

        return $message->user_id == user_id() && $message->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given message.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function destroy(UserPolicyInterface $authUser, Message $message)
    {
        return $message->user_id == user_id() && $message->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given message.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function verify(UserPolicyInterface $authUser, Message $message)
    {
        if ($authUser->canDo('message.message.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given message.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function approve(UserPolicyInterface $authUser, Message $message)
    {
        if ($authUser->canDo('message.message.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $authUser    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($authUser, $ability)
    {
        if ($authUser->isSuperuser()) {
            return true;
        }
    }
}
